# Copyright (c) IFM Lab. All rights reserved.


from typing import Any, Callable, Iterable, TypeVar, Generic, List, Optional, Union
import os
from os import path as osp

from core.registry import DATASETS
# from mmengine.dataset import BaseDataset
from torch.utils.data import Dataset
import numpy as np
import torch

import scipy.io as scio
from torchvision import transforms

from metrics.video_quality_assessment.nn_based.modular_bvqa.ModularBVQA.extract_laplacian.lp_features import exit_folder, VideoDataset, get_features
from metrics.video_quality_assessment.nn_based.modular_bvqa.ModularBVQA.extract_motion.extract_SlowFast_feature_others import slowfast, pack_pathway_output
from metrics.video_quality_assessment.nn_based.modular_bvqa.ModularBVQA.extract_motion.motion_data_loader import VideoDataset_NR_SlowFast_feature

from metrics.video_quality_assessment.nn_based.gstvqa import Test_VQADataset
import h5py


@DATASETS.register_module()
class KONVID1KDataset_ModularBVQA(Dataset):
    """Dataset used in KONVID1KDataset
    Datails in: https://database.mmsp-kn.de/konvid-1k-database.html

    Args:
        video_dir (str): Path to the video directory.
        metadata_dir (str): Path to the metadata directory.
        database (str): Name of the database. Default is 'KoNViD-1k'.
        num_levels (int): Number of levels in the feature hierarchy.
        layer (int): Layer for feature extraction.
        frame_batch_size (int): Batch size for frames.
        rep_dir (str): Directory for representation data.
        datainfo_path (str): Path to dataset information file.
        save_folder (str): Folder to save extracted features.
        imgs_dir (str): Directory containing image frames.
        resize (int): Resize dimension for frames.
        num_frame (int): Number of frames per video.
        num_workers (int): Number of workers for DataLoader.
        feature_save_folder (str): Directory to save extracted motion features.
    """

    def __init__(self,
                 video_dir='',
                 metadata_dir='',
                 database='KoNViD-1k',
                 num_levels=6,
                 layer=2,
                 frame_batch_size=10,
                 rep_dir='',
                 datainfo_path='data/KoNViD-1k_data.mat',
                 save_folder='/data/konvid_1k/konvid1k_LP_ResNet18/',
                 imgs_dir='data/konvid1k_image_all_fps1',
                 resize=224,
                 num_frame=32,
                 num_workers=2,
                 feature_save_folder=''):
        super().__init__()

        self.video_dir = video_dir
        self.metadata_dir = metadata_dir
        self.database = database
        self.num_levels = num_levels
        self.layer = layer
        self.frame_batch_size = frame_batch_size
        self.rep_dir = rep_dir
        self.datainfo_path = datainfo_path
        self.save_folder = os.getcwd() + save_folder
        self.imgs_dir = imgs_dir
        self.resize = resize
        self.num_frame = num_frame
        self.num_workers = num_workers
        self.feature_save_folder = feature_save_folder

        # Prepare Spatial Rectifier
        dataInfo = scio.loadmat(os.getcwd() + self.rep_dir + self.datainfo_path)
        n_video = len(dataInfo['video_names'])
        video_names = []

        for i in range(n_video):
            video_names.append(dataInfo['video_names'][i][0][0].split('_')[0])
        video_length_read = 8

        device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

        dataset = VideoDataset(
            database_name=self.database, 
            videos_dir=os.getcwd()+self.rep_dir+self.imgs_dir, 
            video_names=video_names, 
            num_levels=self.num_levels
        )

        for i in range(len(dataset)):
            print(i)
            current_data, video_name_str = dataset[i]
            features = get_features(current_data, self.layer, self.frame_batch_size, device)
            exit_folder(os.path.join(save_folder, video_name_str))
            for j in range(video_length_read):
                img_features = features[j*(self.num_levels-1) : (j+1)*(self.num_levels1)]
                np.save(os.path.join(save_folder, video_name_str, '{:03d}'.format(j)), img_features.to('cpu').numpy())
        
        # Prepare Temporal Rectifier
        transformations = transforms.Compose([transforms.Resize([self.resize, self.resize]), transforms.ToTensor(),
                                          transforms.Normalize(mean=[0.45, 0.45, 0.45],
                                                               std=[0.225, 0.225, 0.225])])
        trainset = VideoDataset_NR_SlowFast_feature(
            database_name=self.database, 
            data_dir=os.getcwd()+self.video_dir, 
            filename_path=os.getcwd()+self.rep_dir+self.datainfo_path, 
            transform=transformations, 
            resize=self.resize, 
            num_frame=self.num_frame)
       
        train_loader = torch.utils.data.DataLoader(trainset, batch_size=1,
                                               shuffle=False, num_workers=self.num_workers)

        device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        model = slowfast()
        model = model.to(device)
        # do validation after each epoch
        with torch.no_grad():
            model.eval()
            for i, (video, mos, video_name) in enumerate(train_loader):
                video_name = video_name[0]
                print(video_name)
                if not os.path.exists(os.getcwd()+self.feature_save_folder + video_name):
                    os.makedirs(os.getcwd()+self.feature_save_folder + video_name)

                for idx, ele in enumerate(video):
                    # ele = ele.to(device)
                    ele = ele.permute(0, 2, 1, 3, 4)
                    inputs = pack_pathway_output(
                        frames=ele, 
                        device=device)
                    slow_feature, fast_feature = model(inputs)
                    np.save(os.getcwd()+self.feature_save_folder + video_name + '/' + 'feature_' + str(idx) + '_slow_feature',
                            slow_feature.to('cpu').numpy())
                    np.save(os.getcwd()+self.feature_save_folder + video_name + '/' + 'feature_' + str(idx) + '_fast_feature',
                            fast_feature.to('cpu').numpy())

        # Prepare Base Quality Predictor
        dataInfo = scio.loadmat(filename_path)
        n_video = len(dataInfo['video_names'])
        video_names = []

        for i in range(n_video):
            video_names.append(dataInfo['video_names'][i][0][0])

        save_folder = 'data/konvid1k_image_all_fps1'
        for i in range(n_video):
            video_name = video_names[i]
            v_name = video_name.split('_')[0]+'.mp4'
            print('start extract {}th video: {}'.format(i, video_name))
            extract_frame(videos_dir, v_name, save_folder)

    def __len__(self):
        return self.dataset.__len__()
    
    def __getitem__(self, idx) -> Any:
        return self.dataset.__getitem__(idx)
        


import json
import logging
import os
import random
import re
import time
import traceback
import warnings
from io import BytesIO
import pandas as pd
import h5py
import numpy as np
import torch
from icecream import ic
from PIL import Image, ImageFile
from torch.utils.data import Dataset, Subset

from utils import get_args

ImageFile.LOAD_TRUNCATED_IMAGES = True
ImageFile.MAX_IMAGE_PIXELS = None
Image.MAX_IMAGE_PIXELS = None

logging.basicConfig(format='%(asctime)s - %(levelname)s - %(name)s -   %(message)s',
                    datefmt='%m/%d/%Y %H:%M:%S',
                    level=logging.INFO)
warnings.filterwarnings("ignore")
logger = logging.getLogger(__name__)


def load_jsonl(filename):
    with open(filename, "r", encoding="utf-8") as f:
        return [json.loads(l.strip("\n")) for l in f.readlines()]


class MultiModalDataset(Dataset):
    """MultiModal dataset"""

    def __init__(self, input_file, tokenizer, processor,
                 max_length=2048,
                 media_tokens=['<image>', '<|video|>'], loss_objective = 'sequential'):
        
        args = get_args()

        self.loss_objective = loss_objective
        if 'sequential' in self.loss_objective:
            self.dataset = pd.read_csv(input_file) 
            self.dataset = self.dataset.dropna()
        else:
            raise NotImplementedError('dataset loader not implemented for other loss objectives')
        
        self.dataset = pd.read_csv(input_file)
        self.tokenizer = tokenizer
        self.max_length = max_length
        self.processor = processor
        self.media_tokens = {k: -int(i+1) for i, k in enumerate(media_tokens)}
        self.media_lengths = {'<image>': 1+64,'<|video|>': 1+64}
        print("num_media_token: ", self.media_lengths)
        print(len(self.dataset))
        self.bucket = {}

    def __len__(self):
        return len(self.dataset)

    def __getitem__(self, index):
        
        data = self.dataset.iloc[index]
        videopath = data['videopath']
        caption   = data['caption']
        # video_input = self.processor(videos=[videopath], num_frames=16, return_tensors='pt') # video_pixel_values
        video_input = self.processor(videos=[videopath], num_frames=32, return_tensors='pt') # video_pixel_values
        text_input = self._extract_text_token_from_conversation(caption, self.max_length, index)
        item  = {'video': video_input, 'text': text_input, 'videopath': videopath, 'caption': caption}
        return item

    def _extract_text_token_from_conversation(self, data, max_length, index):
        # output enc_chunk
        enc_chunk = []

        if self.tokenizer.bos_token_id > 0:
            prompt_chunk = [self.tokenizer.bos_token_id]
        else:
            prompt_chunk = []

        # conversation = data["completion"]
        conversation = data

        # For Text only data
        if all([media_token not in conversation for media_token in self.media_tokens.keys()]):
            pattern = '|'.join(map(re.escape, ['AI: ', '\nHuman: ']))
            chunk_strs = re.split(f'({pattern})', conversation)
            prompt_length = -1
            stop_flag = False
            for idx, chunk_str in enumerate(chunk_strs):
                if idx == 0:
                    enc_chunk = prompt_chunk + \
                        self.tokenizer(chunk_str, add_special_tokens=False)[
                            'input_ids']
                    enc_length = len(enc_chunk)
                    label_chunk = [0] * enc_length
                else:
                    if chunk_strs[idx-1] == 'AI: ':
                        curr_chunk = self.tokenizer(
                            chunk_str, add_special_tokens=False)['input_ids']
                        if enc_length + len(curr_chunk) >= max_length:
                            curr_chunk = curr_chunk[:max_length-enc_length]
                            stop_flag = True
                        curr_chunk += [self.tokenizer.eos_token_id]
                        enc_length += len(curr_chunk)
                        enc_chunk += curr_chunk
                        label_chunk += [1] * len(curr_chunk)
                    else:
                        curr_chunk = self.tokenizer(
                            chunk_str, add_special_tokens=False)['input_ids']
                        if enc_length + len(curr_chunk) >= max_length + 1:
                            curr_chunk = curr_chunk[:max_length+1-enc_length]
                            stop_flag = True
                        enc_length += len(curr_chunk)
                        enc_chunk += curr_chunk
                        label_chunk += [0] * len(curr_chunk)
                    if stop_flag:
                        break

        # For Image-Text Data
        else:
            enc_length = 0
            prompt_length = -2
            pattern = '|'.join(
                map(re.escape, list(self.media_tokens.keys()) + ['AI: ', '\nHuman: ']))
            chunk_strs = re.split(f'({pattern})', conversation)
            chunk_strs = [x for x in chunk_strs if len(x) > 0]
            for idx, chunk_str in enumerate(chunk_strs):
                if enc_length >= max_length + 1:
                    break

                if idx == 0:
                    enc_chunk = prompt_chunk + \
                        self.tokenizer(chunk_str, add_special_tokens=False)[
                            'input_ids']
                    enc_length = len(enc_chunk)
                    label_chunk = [0] * enc_length
                else:
                    if chunk_str in self.media_tokens:
                        # [CLS] + 256 + [EOS]
                        if enc_length + self.media_lengths[chunk_str] > max_length + 1:
                            break
                        else:
                            enc_chunk += [self.media_tokens[chunk_str]
                                          ] * self.media_lengths[chunk_str]
                            enc_length += self.media_lengths[chunk_str]
                            label_chunk += [0] * self.media_lengths[chunk_str]
                    else:

                        if chunk_strs[idx-1] == 'AI: ':
                            curr_chunk = self.tokenizer(
                                chunk_str, add_special_tokens=False)['input_ids']
                            if enc_length + len(curr_chunk) >= max_length:
                                curr_chunk = curr_chunk[:max_length-enc_length]
                            curr_chunk += [self.tokenizer.eos_token_id]
                            enc_length += len(curr_chunk)
                            enc_chunk += curr_chunk
                            label_chunk += [1] * len(curr_chunk)
                        else:
                            curr_chunk = self.tokenizer(
                                chunk_str, add_special_tokens=False)['input_ids']
                            if enc_length + len(curr_chunk) >= max_length + 1:
                                curr_chunk = curr_chunk[:max_length +
                                                        1-enc_length]
                            enc_length += len(curr_chunk)
                            enc_chunk += curr_chunk
                            label_chunk += [0] * len(curr_chunk)

        if enc_length < max_length + 1:
            padding_chunk = [self.tokenizer.pad_token_id] * \
                (max_length + 1 - enc_length)
            padding_length = len(padding_chunk)
            label_chunk += [0] * (max_length + 1 - enc_length)
            enc_chunk = enc_chunk + padding_chunk
        else:
            padding_length = 0

        assert enc_length + padding_length == max_length + \
            1, (index, prompt_length, enc_length,
                padding_length, max_length + 1)
        assert len(label_chunk) == max_length + \
            1, (len(label_chunk), max_length + 1)
        non_padding_mask = [1 if i < enc_length -
                            1 else 0 for i in range(max_length)]

        enc_chunk = torch.tensor(enc_chunk).long()
        non_padding_mask = torch.tensor(non_padding_mask).long()
        prompt_mask = torch.tensor(label_chunk)[1:].long()
        prompt_length = torch.tensor([prompt_length]).long()

        # Create loss mask
        if all([media_token not in conversation for media_token in self.media_tokens.keys()]):
            non_media_mask = torch.ones_like(non_padding_mask).long()
        else:
            tmp_enc_chunk = enc_chunk.clone()
            tmp_enc_chunk[tmp_enc_chunk >= 0] = 1
            tmp_enc_chunk[tmp_enc_chunk < 0] = 0
            non_media_mask = torch.tensor(tmp_enc_chunk).long()
            non_media_mask = non_media_mask[1:].long()
        return {'input_ids': enc_chunk, "prompt_length": prompt_length, 'seq_length': enc_length,
                "non_padding_mask": non_padding_mask, 'non_media_mask': non_media_mask, 'prompt_mask': prompt_mask}


# Copyright (c) IFM Lab. All rights reserved.

from typing import Optional

from core.registry import TRANSFORMS
from mmcv.transforms.base import BaseTransform

from PIL import Image, ImageSequence
from decord import VideoReader
import numpy as np
import torch


@TRANSFORMS.register_module()
class LoadVideoFromFile(BaseTransform):
    """Load a video from file.

    Required Keys:

        - video_path_pd

    Modified Keys:
        - video_pd

    Args:
        height: int, default is -1
            Desired output height of the video, unchanged if `-1` is specified.
        width: int, default is -1
            Desired output width of the video, unchanged if `-1` is specified.
            See details in: https://github.com/dmlc/decord/blob/master/python/decord/video_reader.py#L18
    """
    
    def __init__(self, height: int = -1, width: int = -1):
        self.height = height
        self.width = width


    def transform(self, results: dict) -> Optional[dict]:
        """Functions to load video. 
        Referred to 'https://github.com/Vchitect/VBench/blob/master/vbench/utils.py#L103'

        The function supports loading video in GIF (.gif), PNG (.png), and MP4 (.mp4) formats.
        Depending on the format, it processes and extracts frames accordingly.

        Args:
            results (dict): Result dict from
                :class:`mmengine.dataset.BaseDataset`.

        Returns:
            dict: The dict contains loaded video in shape (F, C, H, W) and 
            meta information if needed. F is the number of frames, C is the 
            number of channels, H is the height, and W is the width.
        
        Raises:
            - NotImplementedError: If the video format is not supported.
        
        The function first determines the format of the video file by its extension.
        For GIFs, it iterates over each frame and converts them to RGB.
        For PNGs, it reads the single frame, converts it to RGB.
        For MP4s, it reads the frames using the VideoReader class and converts them to NumPy arrays.
        If a data_transform is provided, it is applied to the buffer before converting it to a tensor.
        Finally, the tensor is permuted to match the expected (F, C, H, W) format.
        """

        video_path = results['video_path_pd']
        if video_path.endswith('.gif'):
            frame_ls = []
            img = Image.open(video_path)
            for frame in ImageSequence.Iterator(img):
                frame = frame.convert('RGB')
                frame = np.array(frame).astype(np.uint8)
                frame_ls.append(frame)
            buffer = np.array(frame_ls).astype(np.uint8) # (F, H, W, C), np.uint8
        elif video_path.endswith('.png'):
            frame = Image.open(video_path)
            frame = frame.convert('RGB')
            frame = np.array(frame).astype(np.uint8)
            frame_ls = [frame]
            buffer = np.array(frame_ls) # (1, H, W, C), np.uint8
        elif video_path.endswith('.mp4'):
            import decord
            decord.bridge.set_bridge('native')
            if self.width and self.height:
                video_reader = VideoReader(video_path, width=self.width, height=self.height, num_threads=1)
            else:
                video_reader = VideoReader(video_path, num_threads=1)
            frames = video_reader.get_batch(range(len(video_reader)))  # (F, H, W, C), torch.uint8
            buffer = frames.asnumpy().astype(np.uint8) # (F, H, W, C), np.uint8
        else:
            raise NotImplementedError

        frames = torch.Tensor(buffer)
        frames = frames.permute(0, 3, 1, 2) # (F, C, H, W), torch.uint8
        results['video_pd'] = frames

        return results

    def __repr__(self):
        repr_str = (f'{self.__class__.__name__}, '
                    f'height={self.height}, '
                    f'width={self.width}')


# encoding = utf-8

import os
import cv2
import json
import torch
from torch.utils.data import Dataset
from transformers import AutoProcessor
from core.registry import DATASETS

@DATASETS.register_module()
class PickScoreDataset(Dataset):
    def __init__(self, processor_name, video_dir, prompt_dir):
        super(PickScoreDataset, self).__init__()
        self.video_dir = video_dir
        self.prompt_dir = prompt_dir
        self.processor_name = processor_name

        self.processor = AutoProcessor.from_pretrained(self.processor_name)
        self.prompts, self.video_names = self._read_prompt_videoname()

    def _read_prompt_videoname(self):
        with open(self.prompt_dir, 'r') as reader:
            read_data = json.load(reader)
        
        prompt_data_list, video_name_list = [], []
        for item in read_data["datset_list"]:
            prompt = item['prompt_gt'].strip()
            video_name = item['video_path_pd'].strip()
            prompt_data_list.append(prompt)
            video_name_list.append(video_name)

        return prompt_data_list, video_name_list
    

    def __len__(self):
        return len(self.prompts)
    
    def __getitem__(self, index):
        prompt, video_name = self.prompts[index], self.video_names[index]
        video_path = self.video_dir + video_name
        input_frames = []
        cap = cv2.VideoCapture(video_path)
        while cap.isOpened():
            ret, frame = cap.read()
            if not ret:
                break
            frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
            # resized_frame = cv2.resize(frame,(224,224))  # Resize the frame to match the expected input size
            # frames.append(resized_frame)
            input_frames.append(frame)
        input_frame_tensor = self.processor(
            images=input_frames,
            padding=True,
            truncation=True,
            max_length=77,
            return_tensors="pt",
        )['pixel_values']

        input_prompt = self.processor(
            text=[prompt],
            padding=True,
            truncation=True,
            max_length=77,
            return_tensors="pt",
        )['input_ids']
    
        return input_prompt, input_frame_tensor


# DATASETS.register_module(module=PickScoreDataset, force=True)

# if __name__ == '__main__':
#     processor_name_or_path = "laion/CLIP-ViT-H-14-laion2B-s32B-b79K"
#     prompt_dir = 'AIGVE_Tool/data/toy/annotations/evaluate.json'
#     video_dir = 'AIGVE_Tool/data/toy/evaluate/'

#     pickscore_dataset = PickScoreDataset(processor_name=processor_name_or_path,
#                                          video_dir=video_dir,
#                                          prompt_dir=prompt_dir)
    
#     for index, data in enumerate(pickscore_dataset):
#         print(index, data)



# Copyright (c) IFM Lab. All rights reserved.
from mmengine.config import read_base
from mmengine.dataset import DefaultSampler

from datasets.videoscore_dataset import VideoScoreDataset
from metrics.aigve.videoscore.videoscore_metric import VideoScore
import torch

with read_base():
    from ._base_.default import *

val_dataloader = dict(
    batch_size=1,
    drop_last=False,
    sampler=dict(type=DefaultSampler, shuffle=False),
    dataset=dict(
        type=VideoScoreDataset,
        ann_file = 'AIGVE_Tool/data/toy/annotations/evaluate.json',
        data_root='AIGVE_Tool/data/toy/evaluate'
    ),
)

val_evaluator = dict(
    type=VideoScore,
    collect_device='cpu',
    prefix='videoscore',
)


# Copyright (c) IFM Lab. All rights reserved.
from mmengine.config import read_base
from metrics.text_video_alignment.gpt_based import LLMScore


with read_base():
    from ._base_.datasets.toy_dataset import *
    from ._base_.default import *

val_evaluator = dict(
    type= LLMScore,
    openai_key = 'OPENAI_KEY',
)

val_dataloader = dict(
    batch_size=1,
    num_workers=2,
    dataset=dict(
        type=ToyDataset,
        modality=dict(use_video=False, use_text=True, use_image=True),
        image_frame=1,
    )
)


# Copyright (c) IFM Lab. All rights reserved.
from mmengine.config import read_base
from metrics.text_video_alignment.similarity_based import PickScore

with read_base():
    from ._base_.datasets.pickscore_dataset import *
    from ._base_.default import *


val_evaluator = dict(
    type=PickScore,
    model_name = "yuvalkirstain/PickScore_v1",
)

val_dataloader = dict(
    batch_size=4, 
    num_workers=2,
    persistent_workers=True,
    drop_last=False,
    sampler=dict(type=DefaultSampler, shuffle=False),
    dataset=dict(
        type=PickScoreDataset,
        video_dir='AIGVE_Tool/data/toy/evaluate/',
        prompt_dir='AIGVE_Tool/data/toy/annotations/evaluate.json',
        processor_name="laion/CLIP-ViT-H-14-laion2B-s32B-b79K",
    )
)



# Copyright (c) IFM Lab. All rights reserved.

import torch
# from lavis.models import load_model_and_preprocess

from PIL import Image
import requests
from transformers import Blip2Processor, Blip2ForConditionalGeneration
import torch

class GlobalDescriptor:
    def __init__(self) -> None:
        self.device = "cuda" if torch.cuda.is_available() else "cpu"
        # # loads BLIP-2 pre-trained model
        # # LAVIS
        # self.model, self.vis_processors, _ = load_model_and_preprocess(name="blip2_t5", model_type="pretrain_flant5xxl", is_eval=True, device=self.device)

        # HuggingFace
        self.vis_processors = Blip2Processor.from_pretrained("Salesforce/blip2-opt-2.7b")
        self.model = Blip2ForConditionalGeneration.from_pretrained(
            "Salesforce/blip2-opt-2.7b", torch_dtype=torch.float16
        )
        self.model.to(self.device)

    def get_global_description(self, image_src):
        image = Image.open(image_src)
        # # LAVIS
        # # BLIP FLAN
        # raw_image = image.convert("RGB")
        # # prepare the image
        # image = self.vis_processors["eval"](raw_image).unsqueeze(0).to(self.device)
        # generated_txt = self.model.generate({"image": image, "prompt": "Question: what is in this photo? Answer:"})
        # return generated_txt

        # HuggingFace
        inputs = self.vis_processors(images=image, return_tensors="pt").to(self.device, torch.float16)
        generated_ids = self.model.generate(**inputs)
        generated_text = self.vis_processors.batch_decode(generated_ids, skip_special_tokens=True)[0].strip()
        return generated_text


# encoding = utf-8
import os
import sys

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
LAST_SCRIPT_DIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
sys.path.append(os.path.dirname(SCRIPT_DIR))
sys.path.append(os.path.dirname(LAST_SCRIPT_DIR))

import torch
import numpy as np
from core.registry import METRICS
from PIL import Image
from typing import Sequence, Dict

from mmengine.evaluator import BaseMetric
from mmengine.logging import MMLogger
from utils import add_git_submodule, submodule_exists


@METRICS.register_module()
class VIEEvalScore(BaseMetric):
    """ Initialize the ``VIEEvalScore`` evaluator.
    
    Args:
        llm_backbone (str): The name of the LLM model used in the VIEEvalScore evaluator. Defaults to ``got4o``.
        api_key_path (str): The user's api key path to initialize LLM models provides by openai.
        task (str): The task the VIEEvalScore evaluator conducts. Defaults to ''t2v''.
    """
    def __init__(self,
                 llm_backbone: str = "gpt4o",
                 api_key_path: str = 'AIGVE_Tool/metrics/text_video_alignment/gpt_based/VIE/api_key.txt',
                 task: str = 't2v',
                 ):
        super().__init__()
        
        self.api_key_path = api_key_path
        self.llm_backbone = llm_backbone
        self.task = task

        self.submodel_path = 'metrics/text_video_alignment/gpt_based/VIE'
        if not submodule_exists(self.submodel_path):
            add_git_submodule(
                repo_url='https://github.com/TIGER-AI-Lab/VIEScore.git', 
                submodule_path=self.submodel_path
            )  
        self.submodel_path = 'metrics/text_video_alignment/gpt_based/dsg'
        if not submodule_exists(self.submodel_path):
            add_git_submodule(
                repo_url='https://github.com/j-min/DSG.git', 
                submodule_path=self.submodel_path
            )  
        from .VIEScore.viescore import VIEScore 
        from .DSG.dsg.vqa_utils import MPLUG, InstructBLIP

        
        self.vie_score = VIEScore(backbone=self.llm_backbone, task=self.task, key_path=self.api_key_path)
        
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

    
    def process(self, data_batch: Sequence, data_samples: Sequence) -> None:
        """VIEScore process
        Process one batch of data samples and predictions. The processed
        results should be stored in ``self.results``, which will be used to
        compute the metrics when all batches have been processed.

        Args:
            data_batch (Sequence): A batch of data from the dataloader.
            data_samples (Sequence): A batch of data samples that
                contain annotations and predictions.
        """

        result = dict()

        input_prompts, input_videos = data_samples
        bsz = len(input_prompts)

        # Ensure prompt_input is a tensor
        if isinstance(input_prompts, tuple):
            input_prompts = list(input_prompts)
        
        if isinstance(input_videos, tuple):
            input_videos = list(input_videos)
        
        average_vie_score_list = []
        for input_prompt, input_video in zip(input_prompts, input_videos):
            vie_score_list = []
            for index, frame_path in enumerate(input_video):
                pil_image = Image.open(frame_path)
                score_list = self.vie_score.evaluate(pil_image, input_prompt)
                sementics_score, quality_score, overall_score = score_list
                vie_score_list.append(overall_score)
            average_vie_score = sum(vie_score_list)/len(vie_score_list)
            average_vie_score_list.append(average_vie_score)
    
        result['vie_score'] = sum(average_vie_score_list)/len(average_vie_score_list)

        self.results.append(result)


    def compute_metrics(self, results: list) -> Dict[str, float]:
        """Compute the metrics from processed results.

        Args:
            results (list): The processed results of each batch.

        Returns:
            Dict[str, float]: The computed metrics. The keys are the names of
            the metrics, and the values are corresponding results.
        """
        logger: MMLogger = MMLogger.get_current_instance()

        vie_score_np = np.zeros(len(results))
        for i, result in enumerate(results):
            vie_score_np[i] = result['vie_score']
        
        vie_score_np_mean = np.mean(vie_score_np) 

        print("Test results: vie score with dependency={:.4f}"
              .format(vie_score_np_mean))

        return result



# Copyright (c) IFM Lab. All rights reserved.

from typing import Any, Callable, Iterable, TypeVar, Generic, List, Optional, Union
from os import path as osp

from core.registry import DATASETS
from mmengine.dataset import BaseDataset


@DATASETS.register_module()
class ToyDataset(BaseDataset):
    """ToyDataset for testing.

    Args:
        data_root (str, optional): Root directory for data.
        ann_file (str): Annotation file path.
        metainfo (dict, optional): Metadata information.
        data_prefix (dict): Prefix paths for different modalities.
        pipeline (List[Union[Callable, dict]]): Data transformation pipeline.
        modality (dict): Specifies which modalities are used (video, text, image).
        image_frame (int, optional): Number of frames for images.
    """

    def __init__(self,
                 data_root: Optional[str] = None,
                 ann_file: str = '',
                 metainfo: Optional[dict] = None,
                 data_prefix: dict = None,
                 pipeline: List[Union[Callable, dict]] = [],
                 modality: dict = dict(use_video=True, use_text=True, use_image=False),
                 image_frame: int = None,
                 **kwargs) -> None:
        super().__init__(
            data_root=data_root,
            ann_file=ann_file,
            metainfo=metainfo,
            data_prefix=data_prefix,
            pipeline=pipeline,
            **kwargs
        )
        self.modality = modality
        self.image_frame = image_frame
        assert self.modality['use_video'] or self.modality['use_text'], (
            'Please specify the `modality` (`use_video` '
            f', `use_text`) for {self.__class__.__name__}')
        
    def parse_data_info(self, raw_data_info: dict) -> dict:
        """Parse raw data info."""
        info = {}
        info['img_frame'] = None
        if self.modality['use_text']:
            info['prompt_gt'] = osp.join(self.data_prefix.get('video', ''), 
                                         raw_data_info['prompt_gt'])

        if self.modality['use_video'] or self.modality['use_image']:
            info['video_path_pd'] = osp.join(self.data_prefix.get('video', ''), 
                                     raw_data_info['video_path_pd'])
            if self.modality['use_image']:
                info['img_frame'] = self.image_frame
                                     
        return info




# encoding = utf-8

import os
import cv2
import json
import torch
from transformers import AutoProcessor
from torch.utils.data import Dataset
from core.registry import DATASETS

@DATASETS.register_module()
class CLIPTempDataset(Dataset):
    def __init__(self, processor_name, prompt_dir, video_dir):
        super(CLIPTempDataset, self).__init__()
        self.prompt_dir = prompt_dir
        self.video_dir = video_dir
        self.processor_name = processor_name

        self.processor = AutoProcessor.from_pretrained(self.processor_name)
        self.video_names = self._read_videoname()

    def _read_videoname(self):
        with open(self.prompt_dir, 'r') as reader:
            read_data = json.load(reader)
        
        video_name_list = []
        for item in read_data["datset_list"]:
            video_name = item['video_path_pd'].strip()
            video_name_list.append(video_name)

        return video_name_list
    
    def __len__(self):
        return len(self.video_names)-1
    
    def __getitem__(self, index):
        '''return video frame pairs
        '''
        video_name = self.video_names[index]
        video_path = self.video_dir + video_name
        frames = []
        cap = cv2.VideoCapture(video_path)
        while cap.isOpened():
            ret, frame = cap.read()
            if not ret:
                break
            frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
            resized_frame = cv2.resize(frame,(224,224))  # Resize the frame to match the expected input size
            frames.append(resized_frame)

        input_frame_tensor = self.processor(
            images=frames,
            padding=True,
            truncation=True,
            max_length=77,
            return_tensors="pt",
        )['pixel_values']

        return input_frame_tensor


# DATASETS.register_module(module=CLIPTempDataset, force=True)
# if __name__ == '__main__':
#     video_dir = 'AIGVE_Tool/data/toy/evaluate/'
#     clip_dataset = CLIPTempDataset(video_dir=video_dir)
    
#     for index, data in enumerate(clip_dataset):
#         print(index, data)

    





# encoding = utf-8
# code based on: https://github.com/j-min/DSG/tree/main

import os
os.environ["OPENAI_API_KEY"] = ''
import torch
import cv2
import time
import logging
import openai
import numpy as np

from core.registry import METRICS
from copy import deepcopy
from typing import Dict, Optional, Sequence, Union

from mmengine.evaluator import BaseMetric
from mmengine.logging import MMLogger
from functools import lru_cache

# Lazy import to avoid circular import
@lru_cache(maxsize=1)
def lazy_import():
    from utils import add_git_submodule, submodule_exists
    submodel_path = 'metrics/text_video_alignment/gpt_based/dsg'
    if not submodule_exists(submodel_path):
        add_git_submodule(
            repo_url='https://github.com/j-min/DSG.git', 
            submodule_path=submodel_path
        )  
    submodel_path = 'metrics/text_video_alignment/gpt_based/TIFA'
    if not submodule_exists(submodel_path):
        add_git_submodule(
            repo_url='https://github.com/Yushi-Hu/tifa.git', 
            submodule_path=submodel_path
        )   
    from ..dsg.DSG.dsg.openai_utils import openai_completion
    from .tifa.tifascore import get_question_and_answers, filter_question_and_answers, UnifiedQAModel, tifa_score_single, VQAModel
    
    return openai_completion, get_question_and_answers, filter_question_and_answers, UnifiedQAModel, tifa_score_single, VQAModel


@METRICS.register_module()
class TIFAScore(BaseMetric):
    """ Initialize the ``TIFAScore`` evaluator.
    
    Args:   
        openai_key (str): The user's api key of the LLM models openai provides.
        llm_model (str): The name of the LLM model used in the TIFAScore evaluator. Defaults to ``gpt-3.5-turbo``.
        unifiedqa_model_name (str): The name of the ``UnifiedQAModel`` used in TIFAScore evaluator. Defaults to ``allenai/unifiedqa-v2-t5-large-1363200``.
        vqa_model_name (str): The name of the ``VQAModel used`` in TIFAScore evaluator. Defaults to ``mplug-large``.
    """
    def __init__(self, 
                 openai_key,
                 llm_model: str = 'gpt-3.5-turbo',
                 unifiedqa_model_name: str = 'allenai/unifiedqa-v2-t5-large-1363200',
                 vqa_model_name: str = 'mplug-large'):
        super().__init__()
        
        self.openai_key = openai_key
        self.llm_model = llm_model
        self.unifiedqa_model_name = unifiedqa_model_name
        self.openai_completion, self.get_question_and_answers, self.filter_question_and_answers, self.unifiedqa_model, self.tifa_score_single, self.vqa_model = lazy_import()
        self.unifiedqa_model = self.UnifiedQAModel(self.unifiedqa_model_name)
        self.vqa_model_name = vqa_model_name
        self.vqa_model = self.VQAModel(self.vqa_model_name)
        
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

        self.openai_setup()
    
    def openai_setup(self):
        print('set up openai client')
        openai.api_key = self.openai_key
        assert openai.api_key is not None
        test_prompt_string = 'hello, how are you doing?'
        print('test prompt: ', test_prompt_string)
        response = self.openai_completion(
            test_prompt_string,
            model=self.llm_model,
        )
        print('test response: ', response)

    
    def process(self, data_batch: Sequence, data_samples: Sequence) -> None:
        """ TIFAScore process
        Process one batch of data samples and predictions. The processed
        results should be stored in ``self.results``, which will be used to
        compute the metrics when all batches have been processed.

        Args:
            data_batch (Sequence): A batch of data from the dataloader.
            data_samples (Sequence): A batch of data samples that
                contain annotations and predictions.
        """

        result = dict()

        input_prompts, input_videos = data_samples
        bsz = len(input_prompts)

        # Ensure prompt_input is a tensor
        if isinstance(input_prompts, tuple):
            input_prompts = list(input_prompts)
        
        if isinstance(input_videos, tuple):
            input_videos = list(input_videos)
        
        average_tifa_score_list = []
        for input_prompt, input_video in zip(input_prompts, input_videos):
            tifa_score = []
            # Generate questions with GPT-3.5-turbo
            gpt3_questions = self.get_question_and_answers(input_prompt)
            # print(gpt3_questions)
            # Filter questions with UnifiedQA
            filtered_questions = self.filter_question_and_answers(self.unifiedqa_model, gpt3_questions)
            for index, frame_path in enumerate(input_video):
                # calucluate TIFA score
                result = self.tifa_score_single(self.vqa_model, filtered_questions, frame_path)
                # print(result)
                tifa_score.append(result['tifa_score'])
            average_tifa_score = sum(tifa_score)/len(tifa_score)
            average_tifa_score_list.append(average_tifa_score)
    
        result['tifa_score'] = sum(average_tifa_score_list)/len(average_tifa_score_list)

        self.results.append(result)


    def compute_metrics(self, results: list) -> Dict[str, float]:
        """Compute the metrics from processed results.

        Args:
            results (list): The processed results of each batch.

        Returns:
            Dict[str, float]: The computed metrics. The keys are the names of
            the metrics, and the values are corresponding results.
        """
        logger: MMLogger = MMLogger.get_current_instance()

        tifa_score_np = np.zeros(len(results))
        for i, result in enumerate(results):
            tifa_score_np[i] = result['tifa_score']
        
        tifa_score_np_mean = np.mean(tifa_score_np) 

        print("Test results: tifa score={:.4f}"
              .format(tifa_score_np_mean))

        return result



# Copyright (c) IFM Lab. All rights reserved.

from mmengine.model import BaseModel
from core import MODELS
import math

@MODELS.register_module()
class VQAModel(BaseModel):
    
    def __init__(self, init_cfg=None):
        super().__init__(init_cfg)
    
    def forward(self, x):
        return x



# Copyright (c) IFM Lab. All rights reserved.

import os, sys
import cv2
import json
import torch
import numpy as np
from torch.utils.data import Dataset
from torchvision import transforms
from PIL import Image
import torch.nn as nn
from functools import lru_cache
from core.registry import DATASETS

# Lazy import to avoid circular import
@lru_cache(maxsize=1)
def lazy_import():
    from metrics.video_quality_assessment.nn_based.lightvqa_plus.Light_VQA_plus.extract_temporal_features import slowfast, pack_pathway_output
    return slowfast, pack_pathway_output


@DATASETS.register_module()
class FidDataset(Dataset):
    """
    Dataset for Fréchet Inception Distance (FID) evaluation.

    For each sample, this dataset:
        - Loads both the ground-truth (real) and generated (predicted) videos.
        - Converts each video into a tensor of shape [T, C, H, W] using OpenCV.
        - Optionally pads or truncates videos to a fixed number of frames.

    Args:
        video_dir (str): Directory containing video files.
        prompt_dir (str): Path to JSON file that lists ground-truth and generated video filenames.
        max_len (int): Maximum number of frames to load per video. Default: 500.
        if_pad (bool): Whether to pad videos to exactly `max_len` frames. If False, videos can have variable lengths.
    """

    def __init__(self, video_dir, prompt_dir, max_len=500, if_pad=False):
        super(FidDataset, self).__init__()
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        self.video_dir = video_dir
        self.prompt_dir = prompt_dir
        self.max_len = max_len
        self.if_pad = if_pad

        self.gt_video_names, self.gen_video_names = self._read_video_names()

    def _read_video_names(self):
        """Reads video names from the dataset JSON file."""
        with open(self.prompt_dir, 'r') as reader:
            read_data = json.load(reader)
            gt_video_names = [item['video_path_gt'].strip() for item in read_data["data_list"]]
            gen_video_names = [item['video_path_pd'].strip() for item in read_data["data_list"]]
        return gt_video_names, gen_video_names
    
    def _load_video_tensor(self, video_path: str) -> torch.Tensor:
        """Load a video and return its tensor of shape [T, C, H, W]."""
        assert os.path.exists(video_path), f"Video file not found: {video_path}"
        cap = cv2.VideoCapture(video_path)
        input_frames = []
        frame_count = 0
        while cap.isOpened() and frame_count < self.max_len:
            ret, frame = cap.read()
            if not ret:
                break
            frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
            input_frames.append(torch.tensor(frame).float())
            frame_count += 1

        cap.release()

        if len(input_frames) == 0:
            raise RuntimeError(f"No valid frames found in {video_path}")

        if self.if_pad:
            num_frames = len(input_frames)
            if num_frames < self.max_len:
                pad_frames = torch.zeros((self.max_len - num_frames, *input_frames[0].shape))
                video_tensor = torch.cat((torch.stack(input_frames), pad_frames), dim=0)
            else:
                video_tensor = torch.stack(input_frames[:self.max_len])
        else:
            video_tensor = torch.stack(input_frames)

        # Convert from [T, H, W, C] to [T, C, H, W]
        return video_tensor.permute(0, 3, 1, 2)
    
    def __len__(self):
        return len(self.gt_video_names)

    def __getitem__(self, index) -> tuple[torch.Tensor, torch.Tensor, str, str]:
        """
        Returns:
            Tuple[torch.Tensor, torch.Tensor, str, str]: 
                - Ground-truth (Real) video tensor of shape [T, C, H, W].
                - Generated video tensor of shape [T, C, H, W].
                - Ground-truth video name.
                - Generated video name.
        """
        gt_video_name = self.gt_video_names[index]
        gt_video_path = os.path.join(self.video_dir, gt_video_name)
        gen_video_name = self.gen_video_names[index]
        gen_video_path = os.path.join(self.video_dir, gen_video_name) 

        gt_video_tensor = self._load_video_tensor(gt_video_path)
        gen_video_tensor = self._load_video_tensor(gen_video_path)

        return gt_video_tensor, gen_video_tensor, gt_video_name, gen_video_name
   



# Copyright (c) IFM Lab. All rights reserved.
from mmengine.config import read_base
from metrics.text_video_alignment.gpt_based import DSGScore

with read_base():
    from ._base_.datasets.dsg_dataset import *
    from ._base_.default import *

openai_key = ''

val_evaluator = dict(
    type=DSGScore,
    vqa_model_name = "InstructBLIP",
)

val_dataloader = dict(
    batch_size=1, 
    num_workers=2,
    persistent_workers=True,
    drop_last=False,
    sampler=dict(type=DefaultSampler, shuffle=False),
    dataset=dict(
        type=DSGDataset,
        openai_key=openai_key,
        video_dir='AIGVE_Tool/data/toy/evaluate/',
        prompt_dir='AIGVE_Tool/data/toy/annotations/evaluate.json',
    )
)



# Copyright (c) IFM Lab. All rights reserved.

import os, json
from typing import Dict, Sequence
import torch
import torch.nn as nn
import torchvision.transforms as transforms
import torchvision.models as models
import numpy as np
from mmengine.evaluator import BaseMetric
from mmengine.registry import METRICS



@METRICS.register_module()
class ISScore(BaseMetric):
    """
    Inception Score (IS) implementation.
    
    The Inception Score measures the quality and diversity of generated images
    by evaluating the KL divergence between the conditional class distribution
    and the marginal class distribution.
    
    Args:
        model_name (str): Name of the model to use. Currently only 'inception_v3' is supported.
        input_shape (tuple): Input shape for the model (height, width, channels).
        splits (int): Number of splits to use when calculating the score.
        is_gpu (bool): Whether to use GPU. Defaults to True.
    """

    def __init__(
            self, 
            model_name: str = 'inception_v3', 
            input_shape: tuple = (299, 299, 3), 
            splits: int = 10,
            is_gpu: bool = True):
        super(ISScore, self).__init__()
        self.device = torch.device("cuda" if is_gpu and torch.cuda.is_available() else "cpu")
        self.splits = splits

        if model_name == 'inception_v3':
            self.model = models.inception_v3(pretrained=True, transform_input=False)
            self.model.fc = nn.Identity()  # Remove classification head
            self.model.eval().to(self.device)
        else:
            raise ValueError(f"Model '{model_name}' is not supported for Inception Score computation.")
        
        self.transform = transforms.Compose([
            transforms.Resize((input_shape[0], input_shape[1])),  # InceptionV3 input size
            transforms.Normalize(mean=[0.5, 0.5, 0.5], std=[0.5, 0.5, 0.5])  # Normalize to [-1, 1]
        ])

    def preprocess_tensor(self, images: torch.Tensor) -> torch.Tensor:
        """
        Resize and normalize images.

        Args:
            images (torch.Tensor): Tensor of shape [B, C, H, W].

        Returns:
            torch.Tensor: Preprocessed images.
        """
        return self.transform(images / 255.0)

    def compute_inception_features(self, images: torch.Tensor) -> torch.Tensor:
        """
        Compute Inception features for a batch of images.

        Args:
            images (torch.Tensor): Preprocessed image tensor.

        Returns:
            torch.Tensor: Feature activations from InceptionV3.
        """
        images = self.preprocess_tensor(images).to(self.device)
        with torch.no_grad():
            features = self.model(images).cpu()
        return features

    def calculate_is(self, images: torch.Tensor) -> tuple[float, float]:
        """
        Compute Inception Score.

        Args:
            images (torch.Tensor): Image tensor [B, C, H, W].

        Returns:
            Tuple[float, float]: Mean and standard deviation of Inception Score.
        """
        features = self.compute_inception_features(images)
        preds = torch.nn.functional.softmax(features, dim=1).numpy()

        # Split into chunks
        scores = []
        batch_size = preds.shape[0]
        split_size = batch_size // self.splits

        for i in range(self.splits):
            part = preds[i * split_size:(i + 1) * split_size]
            kl = part * (np.log(part) - np.log(np.expand_dims(np.mean(part, axis=0), 0)))
            kl = np.mean(np.sum(kl, axis=1))
            scores.append(np.exp(kl))

        return float(np.mean(scores)), float(np.std(scores))
    
    def process(self, data_batch: dict, data_samples: Sequence[dict]) -> None:
        """
        Process one batch of data samples and compute IS.

        Args:
            data_batch (dict): A batch of data from the dataloader (not used here).
            data_samples (List[Tuple[torch.Tensor], Tuple[torch.Tensor], Tuple[str], Tuple[str]]):
                A list containing two tuples:
                - A tuple of `real_tensor` (torch.Tensor): Real video tensor [T, C, H, W].
                - A tuple of `gen_tensor` (torch.Tensor): Generated video tensor [T, C, H, W].
                - A tuple of `real_video_name` (str): Ground-truth video filename.
                - A tuple of `gen_video_name` (str): Generated video filename.
                The len of each tuples are the batch size.
        """
        results = []
        real_tensor_tuple, gen_tensor_tuple, real_video_name_tuple, gen_video_name_tuple = data_samples

        batch_size = len(gen_tensor_tuple)
        with torch.no_grad():
            for i in range(batch_size):
                real_video_name = real_video_name_tuple[i]
                gen_video_name = gen_video_name_tuple[i]
                real_tensor = real_tensor_tuple[i]
                gen_tensor = gen_tensor_tuple[i]
                is_score, is_std = self.calculate_is(gen_tensor)

                results.append({
                    "Real video_name": real_video_name, 
                    "Generated video_name": gen_video_name, 
                    "IS_Score": is_score,
                    "IS_Std": is_std
                })
                print(f"Processed IS score {is_score:.4f} ± {is_std:.4f} for {gen_video_name}")

        self.results.extend(results)

    def compute_metrics(self, results: list) -> Dict[str, float]:
        """
        Compute the final IS score.

        Args:
            results (list): List of IS scores for each batch.

        Returns:
            Dict[str, float]: Dictionary containing mean IS score and standard deviation.
        """
        scores = np.array([res["IS_Score"] for res in self.results])
        stds = np.array([res["IS_Std"] for res in self.results])

        mean_score = np.mean(scores) if scores.size > 0 else 0.0
        mean_std = np.mean(stds) if stds.size > 0 else 0.0

        print(f"IS mean score: {mean_score:.4f} ± {mean_std:.4f}")

        json_file_path = os.path.join(os.getcwd(), "is_results.json")
        final_results = {
            "video_results": self.results, 
            "IS_Mean_Score": mean_score, 
            "IS_Mean_Std": mean_std
        }
        with open(json_file_path, "w") as json_file:
            json.dump(final_results, json_file, indent=4)
        print(f"IS mean score saved to {json_file_path}")

        return {"IS_Mean_Score": mean_score, "IS_Mean_Std": mean_std}

    


from mmengine.config import read_base
from mmengine.dataset import DefaultSampler
from metrics.video_quality_assessment.nn_based.gstvqa.gstvqa_metric import GSTVQA
from datasets import GSTVQADataset

with read_base():
    from ._base_.default import *

val_dataloader = dict(
    batch_size=1,
    num_workers=4,
    persistent_workers=True,
    drop_last=False,
    sampler=dict(type=DefaultSampler, shuffle=False),
    dataset=dict(
        type=GSTVQADataset,
        # video_dir='AIGVE_Tool/aigve/data/toy/evaluate/', # it has 16 frames for each video, each frame is [512, 512, 3]
        # prompt_dir='AIGVE_Tool/aigve/data/toy/annotations/evaluate.json',
        video_dir='AIGVE_Tool/aigve/data/AIGVE_Bench/videos_3frame/', # it has 81 frames for each video, each frame is [768, 1360, 3]
        prompt_dir='AIGVE_Tool/aigve/data/AIGVE_Bench/annotations/test.json',
        model_name='vgg16',  # User can choose 'vgg16' or 'resnet18'
        max_len=3,
    )
)

val_evaluator = dict(
    type=GSTVQA,
    model_path="metrics/video_quality_assessment/nn_based/gstvqa/GSTVQA/TCSVT_Release/GVQA_Release/GVQA_Cross/models/training-all-data-GSTVQA-konvid-EXP0-best",
)



# Copyright (c) IFM Lab. All rights reserved.

import argparse, logging, os
import os.path as osp

from mmengine.config import Config, DictAction
from mmengine.logging import print_log
from mmengine.registry import RUNNERS
from mmengine.runner import Runner
import torch.multiprocessing as mp


def parse_args():
    parser = argparse.ArgumentParser(description="VQA Toolkit")
    parser.add_argument("config", help="evaluation metric config file path")
    parser.add_argument("--work-dir", help="the dir to save logs")
    args = parser.parse_args()
    return args

def main():
    args = parse_args()

    print("CFG: ", args.config)
    cfg = Config.fromfile(args.config)

    # work_dir is determined in this priority: CLI > segment in file > filename
    if args.work_dir is not None:
        cfg.work_dir = args.work_dir
    elif cfg.get('work_dir', None) is None:
        cfg.work_dir = osp.join('./work_dirs',
                                osp.splitext(osp.basename(args.config))[0])

    # build the runner from config
    if 'runner_type' not in cfg:
        runner = Runner.from_cfg(cfg)
    else:
        runner = RUNNERS.build(cfg)

    runner.val()


if __name__ == '__main__':
    mp.set_start_method('spawn', force=True)
    main()


# Copyright (c) IFM Lab. All rights reserved.

from mmengine.config import read_base
from mmengine.dataset import DefaultSampler
from metrics.video_quality_assessment.distribution_based.fid_metric import FIDScore
from datasets import FidDataset

with read_base():
    from ._base_.default import *

val_dataloader = dict(
    batch_size=2,  
    num_workers=1,
    persistent_workers=True,
    drop_last=False,
    sampler=dict(type=DefaultSampler, shuffle=False),
    dataset=dict(
        type=FidDataset,
        video_dir='/home/xinhao/AIGVE_Tool/aigve/data/toy/evaluate/',  
        prompt_dir='/home/xinhao/AIGVE_Tool/aigve/data/toy/annotations/evaluate.json',   
        max_len=20,
        if_pad=False
    )
)

val_evaluator = dict(
    type=FIDScore,
    model_name='inception_v3',  # The model used for FID calculation (commonly InceptionV3)
    input_shape=(299, 299, 3),  # Image input size for InceptionV3
    is_gpu = True
)



# Copyright (c) IFM Lab. All rights reserved.
# from tensorflow.keras.applications.inception_v3 import InceptionV3, preprocess_input
import torch
import torch.nn as nn
import torchvision.models as models
import torchvision.transforms as transforms
from mmengine.evaluator import BaseMetric
from mmengine.registry import METRICS
import numpy as np
from scipy.linalg import sqrtm
from typing import Dict, Sequence
import json, os


@METRICS.register_module()
class FIDScore(BaseMetric):

    def __init__(self, 
                 model_name: str = 'inception_v3', 
                 input_shape: tuple = (299, 299, 3), 
                 is_gpu: str = True):
        super(FIDScore, self).__init__()
        self.device = torch.device("cuda" if is_gpu else "cpu")
        self.model_name = model_name
        self.input_shape = input_shape
        if self.model_name == "inception_v3":
            self.model = models.inception_v3(pretrained=True, transform_input=False)
            self.model.fc = nn.Identity()  # Remove classification head
            self.model.eval().to(self.device)
        else:
            raise ValueError(f"Model '{self.model_name}' is not supported for FID computation.")

        # Define preprocessing for InceptionV3
        self.transform = transforms.Compose([
            transforms.Resize((self.input_shape[0], self.input_shape[1])),  # InceptionV3 input size
            transforms.Normalize(mean=[0.5, 0.5, 0.5], std=[0.5, 0.5, 0.5])  # Normalize to [-1, 1]
        ])
        
    def preprocess_tensor(self, video_tensor: torch.Tensor) -> torch.Tensor:
        """
        Resize and normalize a video tensor.

        Args:
            video_tensor (torch.Tensor): Tensor of shape [T, C, H, W].

        Returns:
            torch.Tensor: Preprocessed tensor of shape [T, C, H, W].
        """
        video_tensor = self.transform(video_tensor / 255.0)
        return video_tensor

    def calculate_statistics(self, video_tensor: torch.Tensor) -> tuple[np.ndarray, np.ndarray]:
        """
        Calculate activation statistics (mean and covariance) from video frames.

        Args:
            video_tensor (torch.Tensor): Video tensor [T, C, H, W].

        Returns:
            Tuple of mean and covariance matrix.
        """
        video_tensor = self.preprocess_tensor(video_tensor).to(self.device)
        with torch.no_grad():
            features = self.model(video_tensor).cpu().numpy()  # Extract 2048-d feature vectors

        mu = features.mean(axis=0)
        sigma = np.cov(features, rowvar=False)
        return mu, sigma

    def calculate_fid(self, real: torch.Tensor, fake: torch.Tensor) -> float:
        """
        Calculate FID score between real and generated videos.

        Args:
            real (torch.Tensor): Real video tensor [T, C, H, W].
            fake (torch.Tensor): Generated video tensor [T, C, H, W].

        Returns:
            float: FID score.
        """
        mu1, sigma1 = self.calculate_statistics(real) # Shape[2048], Shape[2048, 2048]
        mu2, sigma2 = self.calculate_statistics(fake)

        # Compute FID score
        ssdiff = np.sum((mu1 - mu2) ** 2.0)
        covmean = sqrtm(sigma1 @ sigma2)

        # Check and correct for imaginary numbers
        if np.iscomplexobj(covmean):
            covmean = covmean.real

        fid = ssdiff + np.trace(sigma1 + sigma2 - 2.0 * covmean)
        return fid


    def process(self, data_batch: dict, data_samples: Sequence[dict]) -> None:
        """
        Process one batch of data samples and compute FID.

        Args:
            data_batch (dict): A batch of data from the dataloader (not used here).
            data_samples (List[Tuple[torch.Tensor], Tuple[torch.Tensor], Tuple[str], Tuple[str]]):
                A list containing two tuples:
                - A tuple of `real_tensor` (torch.Tensor): Real video tensor [T, C, H, W].
                - A tuple of `gen_tensor` (torch.Tensor): Generated video tensor [T, C, H, W].
                - A tuple of `real_video_name` (str): Ground-truth video filename.
                - A tuple of `gen_video_name` (str): Generated video filename.
                The len of each tuples are the batch size.
        """
        results = []
        real_tensor_tuple, gen_tensor_tuple, real_video_name_tuple, gen_video_name_tuple = data_samples

        batch_size = len(real_tensor_tuple)
        with torch.no_grad():
            for i in range(batch_size):
                real_video_name = real_video_name_tuple[i]
                gen_video_name = gen_video_name_tuple[i]
                real_tensor = real_tensor_tuple[i]
                gen_tensor = gen_tensor_tuple[i]
                fid_score = self.calculate_fid(real_tensor, gen_tensor)

                results.append({
                    "Real video_name": real_video_name, 
                    "Generated video_name": gen_video_name, 
                    "FID_Score": fid_score
                })
                print(f"Processed score {fid_score:.4f} between {real_video_name} and {gen_video_name}")

        self.results.extend(results)

    def compute_metrics(self, results: list) -> Dict[str, float]:
        """Compute the final FID score."""
        scores = np.array([res["FID_Score"] for res in self.results])
        mean_score = np.mean(scores) if scores.size > 0 else 0.0
        print(f"FID mean score: {mean_score:.4f}")

        json_file_path = os.path.join(os.getcwd(), "fid_results.json")
        final_results = {
            "video_results": self.results, 
            "FID_Mean_Score": mean_score
        }
        with open(json_file_path, "w") as json_file:
            json.dump(final_results, json_file, indent=4)
        print(f"FID mean score saved to {json_file_path}")
        
        return {'FID_Mean_Score': mean_score}


# Copyright (c) IFM Lab. All rights reserved.

from typing import Dict, List, Optional, Sequence, Union
from mmengine.evaluator import BaseMetric
from core.registry import METRICS
from mmengine.logging import MMLogger
import torch
import torch.nn as nn
import numpy as np

import os
from metrics.video_quality_assessment.nn_based.starvqa import get_cfg
from utils import add_git_submodule, submodule_exists

from .StarVQA.lib.models import ResNet, SlowFast
from .StarVQA.lib.models.vit import vit_base_patch16_224, TimeSformer

@METRICS.register_module()
class StarVQAplus(BaseMetric):
    """The StarVQAplus evaluation metric. https://arxiv.org/pdf/2306.12298
    
    Args:
        collect_device (str): Device used for collecting results from workers.
            Options: 'cpu' and 'gpu'.
        prefix (str, optional): The prefix that will be added in the metric
            names to disambiguate homonymous metrics of different evaluators.
            Default: None.

    Details: https://github.com/GZHU-DVL/CoSTA/blob/main/tools/test_net.py
    """

    def __init__(self,
                 collect_device: str = 'cpu',
                 prefix: Optional[str] = None, 
                 cfg_path:str='',
                 model_name:str='vit_base_patch16_224'
                 ) -> None:
        super().__init__(collect_device=collect_device, prefix=prefix)
        self.cfg_path = os.getcwd() + '/metrics/video_quality_assessment/nn_based/starvqa/' + cfg_path
        self.cfg = get_cfg() # Get default config. See details in https://github.com/GZHU-DVL/CoSTA/blob/main/build/lib/lib/config/defaults.py
        if self.cfg_path is not None:
            self.cfg.merge_from_file(self.cfg_path) # Merge from config file. See details in https://github.com/GZHU-DVL/CoSTA/blob/main/configs/Kinetics/TimeSformer_divST_8x32_224.yaml


        

        self.submodel_path = 'metrics/video_quality_assessment/nn_based/starvqa_plus'
        if not submodule_exists(self.submodel_path):
            add_git_submodule(
                repo_url='https://github.com/GZHU-DVL/CoSTA.git', 
                submodule_path=self.submodel_path
            )
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        self.model_name = model_name
        assert self.model_name in ['vit_base_patch16_224','timesformer','slowfast','resnet']
        if self.model_name == 'vit_base_patch16_224':
            self.model = vit_base_patch16_224(
                cfg=self.cfg,

            ).to(self.device)
        elif self.model_name == 'timesformer':
            self.model = TimeSformer().to(self.device)
        elif self.model_name == 'slowfast':
            self.model = SlowFast().to(self.device)
        elif self.model_name == 'resnet':
            self.model = ResNet().to(self.device)
        else:
            raise NotImplementedError

        
        # self.model.load_state_dict(torch.load('')) # The pretrained model shared from author is out-of-date
        self.model.eval()
        # self.criterion = nn.L1Loss().to(self.device)

        print(f"=====datainfo_path=====: {datainfo_path}")
        Info = h5py.File(name=datainfo_path, mode='r') # Runtime Error otherwise
        self.scale = Info['scores'][0, :].max()  


    # def process(self, data_batch: dict, data_samples: Sequence[dict]) -> None:
    def process(self, data_batch: Sequence, data_samples: Sequence) -> None:
        """GSTVQA process
        Process one batch of data samples and predictions. The processed
        results should be stored in ``self.results``, which will be used to
        compute the metrics when all batches have been processed.

        Args:
            data_batch (Sequence): A batch of data from the dataloader.
            data_samples (Sequence): A batch of data samples that
                contain annotations and predictions.
        """

        result = dict()

        features, length, label, mean_var,std_var,mean_mean,std_mean = data_samples
        # # prompt_gt = data_sample['prompt_gt'] # str
        # video_pd = data_sample['video_pd'] # torch.uint8(F, C, H, W)

        result['y_test'] = self.scale * label.item()

        features = features.to(self.device).float()
        label = label.to(self.device).float()
        mean_var = mean_var.to(self.device).float()
        std_var = std_var.to(self.device).float()
        mean_mean = mean_mean.to(self.device).float()
        std_mean = std_mean.to(self.device).float()

        outputs = self.model(features, length.float(),mean_var,std_var,mean_mean,std_mean)
        result['y_pred'] = self.scale * outputs.item()
        result['loss'] = self.criterion(outputs, label).item()

        self.results.append(result)


    def compute_metrics(self, results: list) -> Dict[str, float]:
        """Compute the metrics from processed results.

        Args:
            results (list): The processed results of each batch.

        Returns:
            Dict[str, float]: The computed metrics. The keys are the names of
            the metrics, and the values are corresponding results.
        """
        logger: MMLogger = MMLogger.get_current_instance()

        assert self.test_index == len(results)
        test_loss = sum(result.get('loss', 0) for result in results) / len(results)
        y_pred_np = np.zeros(len(self.test_index))
        y_test_np = np.zeros(len(self.test_index))
        for i, result in enumerate(results):
            y_pred_np[i] = result['y_pred']
            y_test_np[i] = result['y_test']

        PLCC = stats.pearsonr(y_pred_np, y_test_np)[0]
        SROCC = stats.spearmanr(y_pred_np, y_test_np)[0]
        RMSE = np.sqrt(((y_pred_np-y_test_np) ** 2).mean())
        KROCC = stats.stats.kendalltau(y_pred_np, y_test_np)[0]
        print("Test results: test loss={:.4f}, SROCC={:.4f}, KROCC={:.4f}, PLCC={:.4f}, RMSE={:.4f}"
                .format(test_loss, SROCC, KROCC, PLCC, RMSE))


# Copyright (c) IFM Lab. All rights reserved.

import os
import json
import torch
import torch.nn as nn
import numpy as np
from typing import Dict, Sequence, Tuple
# from scipy import stats
from mmengine.evaluator import BaseMetric
from core.registry import METRICS
from utils import add_git_submodule, submodule_exists

@METRICS.register_module()
class GstVqa(BaseMetric):
    """GstVQA metric modified for the toy dataset. (Supporting 2944-dim features)."""

    def __init__(self, model_path: str):
        super(GstVqa, self).__init__()
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        self.submodel_path = os.path.join(os.getcwd(), 'metrics/video_quality_assessment/nn_based/gstvqa')
        if not submodule_exists(self.submodel_path):
            add_git_submodule(
                repo_url='https://github.com/Baoliang93/GSTVQA.git', 
                submodule_path=self.submodel_path
            )
        from .GSTVQA.TCSVT_Release.GVQA_Release.GVQA_Cross.cross_test import GSTVQA as GSTVQA_model
        self.model = GSTVQA_model().to(self.device)
        self.model.load_state_dict(torch.load(model_path, map_location=self.device))
        self.model.eval()
        # self.criterion = nn.L1Loss().to(self.device)

    def compute_stat_features(self, features: torch.Tensor, num_valid_frames: int) -> Tuple[torch.Tensor]:
        """Compute statistical features mean_var, std_var, mean_mean, std_mean from extracted deep features.

        Args:
            features (torch.Tensor): Tensor of shape [T, 2944].
            num_valid_frames (int): Number of valid frames before padding.

        Returns:
            Tuple[torch.Tensor]: (mean_var, std_var, mean_mean, std_mean), each of shape [1472].
        """
        # Ignore padded frames
        features = features[:num_valid_frames]  # Shape: [num_valid_frames, feature_dim]: [10, 1472]

        if num_valid_frames == 0:  # Edge case: all frames were padded
            return (
                torch.zeros(1472, device=self.device),
                torch.zeros(1472, device=self.device),
                torch.zeros(1472, device=self.device),
                torch.zeros(1472, device=self.device),
            )

        # Split into mean and std components
        mean_features = features[:, :1472]  # First 1472 features are mean-based
        std_features = features[:, 1472:]   # Last 1472 features are std-based

        # Compute per-feature statistics over frames
        mean_mean = mean_features.mean(dim=0)  # Shape: [1472]
        std_mean = std_features.mean(dim=0)    # Shape: [1472]
        mean_var = mean_features.var(dim=0, unbiased=False)  # Shape: [1472]
        std_var = std_features.var(dim=0, unbiased=False)    # Shape: [1472]

        return mean_var, std_var, mean_mean, std_mean

    def process(self, data_batch: Sequence, data_samples: Sequence) -> None:
        """
        Process a batch of extracted deep features for GSTVQA evaluation and store results in a JSON file.

        Args:
            data_batch (SequencTuplee): A batch of data from the dataloader (not used here).
            data_samples (List[ [torch.Tensor], Tuple[int], Tuple[str] ]): 
                A list containing three tuples:
                - A tuple of `deep_features`: Each item is a Tensor of shape [T, 2944]. 
                - A tuple of `num_frames`: Each item is an integer representing the number of valid frames.
                - A tuple of `video_name`: Each item is a string representing the file name for the video.
                The len of each three tuples are the batch size.
        """
        # data_samples an example: [
        #     (tensor([[0., 0., 0.,  ..., 0., 0., 0.],
        #              [0., 0., 0.,  ..., 0., 0., 0.],
        #              ...
        #              [0., 0., 0.,  ..., 0., 0., 0.]]), 
        #      tensor([[0., 0., 0.,  ..., 0., 0., 0.],
        #              [0., 0., 0.,  ..., 0., 0., 0.],
        #              ...
        #              [0., 0., 0.,  ..., 0., 0., 0.]])), 
        #     (10, 10)
        # ]
        results = []
        deep_features_tuple, num_frames_tuple, video_name_tuple = data_samples
        with torch.no_grad():
            for deep_features, num_valid_frames, video_name in zip(deep_features_tuple, num_frames_tuple, video_name_tuple):
                if not isinstance(deep_features, torch.Tensor) or not isinstance(num_valid_frames, int):
                    raise TypeError("Expected deep_features to be a torch.Tensor and num_valid_frames to be an int.")

                if num_valid_frames == 0:  # Edge case: No valid frames
                    results.append({"video_name": 'N/A', "GSTVQA_Score": 0.0})
                    continue

                # Remove padded features
                features = deep_features[:num_valid_frames].to(self.device)

                # Compute statistical features only on valid frames
                mean_var, std_var, mean_mean, std_mean = self.compute_stat_features(features, num_valid_frames)
                mean_var, std_var, mean_mean, std_mean = (
                    mean_var.to(self.device),
                    std_var.to(self.device),
                    mean_mean.to(self.device),
                    std_mean.to(self.device),
                )

                # Length tensor indicating the number of valid frames
                length = torch.tensor([num_valid_frames]).to(self.device)
                # print('features(input) shape', features.unsqueeze(1).shape) # torch.Size([10, 1, 1472])
                # print('input_length shape', length.shape) # torch.Size([1])
                # print('input_length', length) # torch.Size([1])
                # print('mean_mean shape', mean_mean.shape) # torch.Size([1472])
                # print('std_mean shape', std_mean.shape) # torch.Size([1472])
                # print('mean_var shape', mean_var.shape) # torch.Size([1472])
                # print('std_var shape', std_var.shape) # torch.Size([1472])
                
                # Run GSTVQA model
                outputs = self.model(features.unsqueeze(1), length, mean_var, std_var, mean_mean, std_mean)
                score = outputs.item()
                results.append({"video_name": video_name, "GSTVQA_Score": score})
                # print(f"Processed score {score:.4f} for {video_name}")

        self.results.extend(results)


    def compute_metrics(self, results: list) -> Dict[str, float]:
        """Compute final GSTVQA-based metrics."""
        scores = np.array([res['GSTVQA_Score'] for res in self.results])
        mean_score = np.mean(scores)
        print(f"GSTVQA mean score: {mean_score:.4f}")

        json_file_path = os.path.join(os.getcwd(), "gstvqa_results.json")
        final_results = {"video_results": self.results, "GSTVQA_Mean_Score": mean_score}
        with open(json_file_path, "w") as json_file:
            json.dump(final_results, json_file, indent=4)
        print(f"GSTVQA mean score saved to {json_file_path}")

        return {'GSTVQA_Mean_Score': mean_score}


# Copyright (c) IFM Lab. All rights reserved.

from .loading import LoadVideoFromFile 
from .image_reading import read_image_detectron2
from .module_import import add_git_submodule, submodule_exists

__all__ = ['LoadVideoFromFile', 'read_image_detectron2', 'add_git_submodule', 'submodule_exists']


import os.path

from torch.utils.data import Dataset
import json
import re
import torch
from transformers import LlamaTokenizer
from core.registry import DATASETS
from functools import lru_cache

# Lazy import to avoid circular import
@lru_cache(maxsize=1)
def lazy_import_mplug_owl():
    import importlib
    extract_features = importlib.import_module(
        "aigve.metrics.multi_aspect_metrics.videophy.mplug_owl_video"
    )
    return extract_features.MplugOwlImageProcessor, extract_features.MplugOwlProcessor

@DATASETS.register_module()
class VideoPhyDataset(Dataset):
    def __init__(self, data_path, video_root_path, hf_token, tokenizer=None, processor=None, max_length=2048, media_tokens=['<image>', '<|video|>'], hf_checkpoint='videophysics/videocon_physics'):
        """
        Args:
            data_path (str): Path to the data folder, it should be a json file
            tokenizer (Tokenizer): Tokenizer object
            processor (Processor): Processor object
            max_length (int): Maximum length of the input sequence
            media_tokens (list): List of media tokens
        """
        self.dataset = json.load(open(data_path))
        self.video_root_path = video_root_path

        self.hf_token = hf_token
        self.hf_checkpoint = hf_checkpoint
        self.max_length = max_length
        self.media_tokens = {k: -int(i + 1) for i, k in enumerate(media_tokens)}
        self.media_lengths = {'<image>': 1 + 64, '<|video|>': 1 + 64}
        self.bucket = {}


        # initialize tokenizer
        if tokenizer is not None:
            self.tokenizer = tokenizer
        else:
            self.tokenizer = LlamaTokenizer.from_pretrained(self.hf_checkpoint, token=self.hf_token)

        MplugOwlImageProcessor, MplugOwlProcessor = lazy_import_mplug_owl()
        self.image_processor = MplugOwlImageProcessor.from_pretrained(self.hf_checkpoint)
        # initialize processor
        if processor is not None:
            self.processor = processor
        else:
            self.processor = MplugOwlProcessor(self.image_processor, self.tokenizer)

    def __len__(self) -> int:
        """
        Returns:
            int: Length of the dataset
        """
        return self.dataset['metainfo']['length']

    def __getitem__(self, idx):
        """
        Args:
            idx (int): Index of the dataset
        Returns:
            dict: Dictionary containing the video, text, video path and caption
        """
        data = self.dataset['dataset_list'][idx]
        videopath = os.path.join(self.video_root_path, data['video_path_pd'])
        caption = data['prompt_gt']
        # video_input = self.processor(videos=[videopath], num_frames=16, return_tensors='pt') # video_pixel_values
        video_input = self.processor(videos=[videopath], num_frames=32, return_tensors='pt')  # video_pixel_values
        text_input = self._extract_text_token_from_conversation(caption, self.max_length, idx)
        item = {'video': video_input, 'text': text_input, 'videopath': videopath, 'caption': caption}
        return item

    def _extract_text_token_from_conversation(self, data, max_length, index):
        """
        Extracts the text tokens from the conversation
        Args:
            data (str): Conversation
            max_length (int): Maximum length of the input sequence
            index (int): Index of the dataset
        """
        # output enc_chunk
        enc_chunk = []

        if self.tokenizer.bos_token_id > 0:
            prompt_chunk = [self.tokenizer.bos_token_id]
        else:
            prompt_chunk = []

        # conversation = data["completion"]
        conversation = data

        # For Text only data
        if all([media_token not in conversation for media_token in self.media_tokens.keys()]):
            pattern = '|'.join(map(re.escape, ['AI: ', '\nHuman: ']))
            chunk_strs = re.split(f'({pattern})', conversation)
            prompt_length = -1
            stop_flag = False
            for idx, chunk_str in enumerate(chunk_strs):
                if idx == 0:
                    enc_chunk = prompt_chunk + \
                                self.tokenizer(chunk_str, add_special_tokens=False)[
                                    'input_ids']
                    enc_length = len(enc_chunk)
                    label_chunk = [0] * enc_length
                else:
                    if chunk_strs[idx - 1] == 'AI: ':
                        curr_chunk = self.tokenizer(
                            chunk_str, add_special_tokens=False)['input_ids']
                        if enc_length + len(curr_chunk) >= max_length:
                            curr_chunk = curr_chunk[:max_length - enc_length]
                            stop_flag = True
                        curr_chunk += [self.tokenizer.eos_token_id]
                        enc_length += len(curr_chunk)
                        enc_chunk += curr_chunk
                        label_chunk += [1] * len(curr_chunk)
                    else:
                        curr_chunk = self.tokenizer(
                            chunk_str, add_special_tokens=False)['input_ids']
                        if enc_length + len(curr_chunk) >= max_length + 1:
                            curr_chunk = curr_chunk[:max_length + 1 - enc_length]
                            stop_flag = True
                        enc_length += len(curr_chunk)
                        enc_chunk += curr_chunk
                        label_chunk += [0] * len(curr_chunk)
                    if stop_flag:
                        break

        # For Image-Text Data
        else:
            enc_length = 0
            prompt_length = -2
            pattern = '|'.join(
                map(re.escape, list(self.media_tokens.keys()) + ['AI: ', '\nHuman: ']))
            chunk_strs = re.split(f'({pattern})', conversation)
            chunk_strs = [x for x in chunk_strs if len(x) > 0]
            for idx, chunk_str in enumerate(chunk_strs):
                if enc_length >= max_length + 1:
                    break

                if idx == 0:
                    enc_chunk = prompt_chunk + \
                                self.tokenizer(chunk_str, add_special_tokens=False)[
                                    'input_ids']
                    enc_length = len(enc_chunk)
                    label_chunk = [0] * enc_length
                else:
                    if chunk_str in self.media_tokens:
                        # [CLS] + 256 + [EOS]
                        if enc_length + self.media_lengths[chunk_str] > max_length + 1:
                            break
                        else:
                            enc_chunk += [self.media_tokens[chunk_str]
                                          ] * self.media_lengths[chunk_str]
                            enc_length += self.media_lengths[chunk_str]
                            label_chunk += [0] * self.media_lengths[chunk_str]
                    else:

                        if chunk_strs[idx - 1] == 'AI: ':
                            curr_chunk = self.tokenizer(
                                chunk_str, add_special_tokens=False)['input_ids']
                            if enc_length + len(curr_chunk) >= max_length:
                                curr_chunk = curr_chunk[:max_length - enc_length]
                            curr_chunk += [self.tokenizer.eos_token_id]
                            enc_length += len(curr_chunk)
                            enc_chunk += curr_chunk
                            label_chunk += [1] * len(curr_chunk)
                        else:
                            curr_chunk = self.tokenizer(
                                chunk_str, add_special_tokens=False)['input_ids']
                            if enc_length + len(curr_chunk) >= max_length + 1:
                                curr_chunk = curr_chunk[:max_length +
                                                         1 - enc_length]
                            enc_length += len(curr_chunk)
                            enc_chunk += curr_chunk
                            label_chunk += [0] * len(curr_chunk)

        if enc_length < max_length + 1:
            padding_chunk = [self.tokenizer.pad_token_id] * \
                            (max_length + 1 - enc_length)
            padding_length = len(padding_chunk)
            label_chunk += [0] * (max_length + 1 - enc_length)
            enc_chunk = enc_chunk + padding_chunk
        else:
            padding_length = 0

        assert enc_length + padding_length == max_length + \
               1, (index, prompt_length, enc_length,
                   padding_length, max_length + 1)
        assert len(label_chunk) == max_length + \
               1, (len(label_chunk), max_length + 1)
        non_padding_mask = [1 if i < enc_length -
                                 1 else 0 for i in range(max_length)]

        enc_chunk = torch.tensor(enc_chunk).long()
        non_padding_mask = torch.tensor(non_padding_mask).long()
        prompt_mask = torch.tensor(label_chunk)[1:].long()
        prompt_length = torch.tensor([prompt_length]).long()

        # Create loss mask
        if all([media_token not in conversation for media_token in self.media_tokens.keys()]):
            non_media_mask = torch.ones_like(non_padding_mask).long()
        else:
            tmp_enc_chunk = enc_chunk.clone()
            tmp_enc_chunk[tmp_enc_chunk >= 0] = 1
            tmp_enc_chunk[tmp_enc_chunk < 0] = 0
            non_media_mask = torch.tensor(tmp_enc_chunk).long()
            non_media_mask = non_media_mask[1:].long()
        return {'input_ids': enc_chunk, "prompt_length": prompt_length, 'seq_length': enc_length,
                "non_padding_mask": non_padding_mask, 'non_media_mask': non_media_mask, 'prompt_mask': prompt_mask}






# Copyright (c) IFM Lab. All rights reserved.

# from .starvqa_plus_metric import StarVQAplus
# from .StarVQA_PLUS.build.lib.lib.datasets import Kinetics
# from .StarVQA_PLUS.build.lib.lib.config.defaults import get_cfg

# __all__ = ['StarVQAplus', 'Kinetics', 'get_cfg']


# Copyright (c) IFM Lab. All rights reserved.

from typing import Dict, List, Optional, Sequence, Union
from mmengine.evaluator import BaseMetric
from core.registry import METRICS

from mmengine.logging import MMLogger

import os, time
# from .global_descriptor import GlobalDescriptor
# from .local_descriptor import LocalDescriptor
# from utils import read_image_detectron2
# from grit.predictor import VisualizationDemo



@METRICS.register_module()
class LLMScore(BaseMetric):
    """The LLMScore evaluation metric. It is a text-image alignemnet framework.
    
    Args:
        collect_device (str): Device used for collecting results from workers.
            Options: 'cpu' and 'gpu'.
        prefix (str, optional): The prefix that will be added in the metric
            names to disambiguate homonymous metrics of different evaluators.
            Default: None.
    """

    default_prefix: Optional[str] = 'llm_score'

    def __init__(self,
                 collect_device: str = 'cpu',
                 prefix: Optional[str] = None,
                 openai_key: str = None) -> None:
        super().__init__(collect_device=collect_device, prefix=prefix)
        self.openai_key = openai_key

    def process(self, data_batch: dict, data_samples: Sequence[dict]) -> None:
        """LLMScore process
        Process one batch of data samples and predictions. The processed
        results should be stored in ``self.results``, which will be used to
        compute the metrics when all batches have been processed.

        Args:
            data_batch (dict): A batch of data from the dataloader.
            data_samples (Sequence[dict]): A batch of data samples that
                contain annotations and predictions.
        """
        for data_sample in data_samples:
            result = dict()
            prompt_gt = data_sample['prompt_gt']
            video_pd = data_sample['video_pd']
            img_pd = video_pd[data_sample['img_frame']] # torch.uint8(C, H, W)

            openai_key = os.environ['OPENAI_KEY']
            global_descriptor = GlobalDescriptor()
            local_descriptor = LocalDescriptor()

            img = read_image_detectron2(img_pd, format="BGR")
            start_time = time.time()


            result['scores'] = llmscore

            self.results.append(result)




    def compute_metrics(self, results: list) -> Dict[str, float]:
        """Compute the metrics from processed results.

        Args:
            results (list): The processed results of each batch.

        Returns:
            Dict[str, float]: The computed metrics. The keys are the names of
            the metrics, and the values are corresponding results.
        """
        logger: MMLogger = MMLogger.get_current_instance()





# Copyright (c) IFM Lab. All rights reserved.

from mmengine.config import read_base
from mmengine.dataset import DefaultSampler
from metrics.video_quality_assessment.distribution_based.fvd.fvd_metric import FVDScore
from datasets import FidDataset

with read_base():
    from ._base_.default import *


val_dataloader = dict(
    batch_size=2,  
    num_workers=1,
    persistent_workers=True,
    drop_last=False,
    sampler=dict(type=DefaultSampler, shuffle=False),
    dataset=dict(
        type=FidDataset,
        video_dir="/home/xinhao/AIGVE_Tool/aigve/data/toy/evaluate/",
        prompt_dir="/home/xinhao/AIGVE_Tool/aigve/data/toy/annotations/evaluate.json",
        max_len=20,
        if_pad=False
    )
)

val_evaluator = dict(
    type=FVDScore,
    model_path='AIGVE_Tool/aigve/metrics/video_quality_assessment/distribution_based/fvd/model_rgb.pth',  # Path to the I3D model
    feature_layer=-2,  # Feature layer to extract from the I3D model
    is_gpu=True
)



# Copyright (c) IFM Lab. All rights reserved.


from typing import Any, Callable, Iterable, TypeVar, Generic, List, Optional, Union
from os import path as osp

from core.registry import DATASETS
# Importfrom mmengine.dataset import BaseDataset
from torch.utils.data import Dataset


# from metrics.video_quality_assessment.nn_based.gstvqa import Test_VQADataset
import h5py


@DATASETS.register_module()
class GSTVQADatasetCrossData(Dataset):
    """Dataset used in GSTVQA
    Datails in: https://github.com/Baoliang93/GSTVQA

    Args:
        feature_dir (str): Path to the feature directory.
        index (Optional): Index for dataset filtering.
        max_len (int): Maximum sequence length. Default is 500.
        feat_dim (int): Feature dimension. Default is 2944.
        datainfo_path (str): Path to dataset information file.
    """

    def __init__(self,
                 feature_dir, 
                 index=None, 
                 max_len=500, 
                 feat_dim=2944,
                 datainfo_path=''):
        super().__init__()
        self.feature_dir = feature_dir
        self.index = index
        self.max_len = max_len
        self.feat_dim = feat_dim

        self.datainfo_path = datainfo_path
        Info = h5py.File(name=datainfo_path, mode='r') # Runtime Error otherwise
        self.scale = Info['scores'][0, :].max() 

        # See details in https://github.com/Baoliang93/GSTVQA/blob/8463c9c3e5720349606d8efae7a5aa274bf69e7c/TCSVT_Release/GVQA_Release/GVQA_Cross/cross_test.py#L30
        self.dataset = Test_VQADataset( 
            features_dir=self.feature_dir, 
            index=self.index, 
            max_len=self.max_len, 
            feat_dim=self.feat_dim, 
            scale=self.scale
        )

    def __len__(self):
        return self.dataset.__len__()
    
    def __getitem__(self, idx) -> Any:
        return self.dataset.__getitem__(idx)
        


# Copyright (c) IFM Lab. All rights reserved.

from setuptools import setup, find_packages

__version__ = '0.0.1'

requirements = [
    "torch==2.1.0",
    "torchvision==0.16.0",
    "torchaudio==0.12.1",
    "numpy==1.23.5",
    "scipy==1.14.1",
    "pandas==2.2.3",
    "pyyaml==6.0.2",
    "tqdm==4.67.1",
    "matplotlib==3.10.0",

    "mmcv==2.2.0",
    "mmdet==3.3.0",
    "mmengine==0.10.6",

    "transformers==4.46.3",
    "sentencepiece==0.2.0",
    "einops==0.8.1",
    "openai==1.65.1",
    "gdown==5.2.0",
    "fvcore==0.1.5.post20221221",
    "decord==0.6.0",
    "pytorchvideo==0.1.5",
    "clip==1.0",

    # Optional docs tools
    "mkdocs-material==9.6.5",
    "neoteroi-mkdocs==1.1.0",
    "mkdocs-macros-plugin==1.3.7",
    "mkdocs-jupyter==0.25.1",
    "mkdocstrings==0.28.2",
    "mkdocs-rss-plugin==1.17.1",
    "mkdocs-exclude==1.0.2",
    "mkdocstrings-python==1.16.2"
]

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="aigve",  
    version=__version__,
    author="Xinhao Xiang, Xiao Liu, Zizhong Li, Zhuosheng Liu",
    author_email="xhxiang@ucdavis.edu",

    description="A comprehensive and structured evaluation framework for assessing AI-generated video quality.",
    long_description=long_description,
    long_description_content_type="text/markdown",

    url="https://www.aigve.org",
    download_url="https://github.com/ShaneXiangH/AIGVE_Tool",
    packages=find_packages(include=["aigve", "aigve.*"]),

    license="MIT License",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],

    python_requires='>=3.10',
    install_requires=requirements
)



# encoding = utf-8

import os
import torch
import cv2
import time
import logging
import numpy as np

from core.registry import METRICS
from typing import Dict, Optional, Sequence, Union
from transformers import AutoProcessor, CLIPModel

from mmengine.evaluator import BaseMetric
from mmengine.logging import MMLogger
from tqdm import tqdm


@METRICS.register_module()
class CLIPSimScore(BaseMetric):
    """ Initialize the ``CLIPSimScore`` evaluator.
    
    Args:
        processor_name (str): The name of the CLIP processor, which wraps a CLIP feature extractor and a CLIP tokenizer into this single procesor. 
                                Defaults to ``openai/clip-vit-base-patch32``.
        model_name (str): The name of the CLIP model. Defaults to ``openai/clip-vit-base-patch32``.
        logit_scale (bool): Whether to calcualte the cosine similarity as logits. Defaults to False.
    """
    def __init__(self,
                 processor_name: str = "openai/clip-vit-base-patch32",
                 model_name: str = "openai/clip-vit-base-patch32",
                 logit_scale: bool = False,
                #  train_index: int = 4
                 ) -> None:
        super().__init__()
        self.processor_name = processor_name
        self.model_name = model_name
        self.logit_scale = logit_scale

        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        self.processor = AutoProcessor.from_pretrained(self.processor_name)
        self.model = CLIPModel.from_pretrained(self.model_name).to(self.device)
        self.model.eval()

    def process(self, data_batch: Sequence, data_samples: Sequence) -> None:
        """CLIPSimScore process
        Process one batch of data samples and predictions. The processed
        results should be stored in ``self.results``, which will be used to
        compute the metrics when all batches have been processed.

        Args:
            data_batch (Sequence): A batch of data from the dataloader.
            data_samples (Sequence): A batch of data samples that
                contain annotations and predictions.
        """

        result = dict()

        input_prompts, input_videos = data_samples
        bsz = len(input_prompts)

        # Ensure prompt_input is a tensor
        if isinstance(input_prompts, tuple):
            input_prompts = list(input_prompts)
        
        if isinstance(input_videos, tuple):
            input_videos = list(input_videos)
        
        # Initialize an empty list to store each similarity score
        clip_score_sum, clip_score_cnt = 0, 0
        logit_scale = self.model.logit_scale.exp() if self.logit_scale else 1
        with torch.no_grad():
            for input_prompt, input_frames in zip(input_prompts, input_videos):
                input_prompt = input_prompt.to(self.device)
                text_feature = self.model.get_text_features(input_prompt) # [bsz, hid_dim]
                text_feature = text_feature / torch.norm(text_feature, dim=-1, keepdim=True)

                input_frames = input_frames.to(self.device)  # Add batch dimension and move the frame to the device
                frame_feature = self.model.get_image_features(input_frames)
                frame_feature = frame_feature / torch.norm(frame_feature, dim=-1, keepdim=True)

                clip_score = logit_scale * (frame_feature @ text_feature.T).mean().item()
                print('current clip similarity score', clip_score)
                clip_score_sum += clip_score
                clip_score_cnt += 1

        # Calculate the average CLIP score across all frames
        clip_score_videos_avg = clip_score_sum/clip_score_cnt
       
        result['clip_sim_score'] = clip_score_videos_avg

        self.results.append(result)


    def compute_metrics(self, results: list) -> Dict[str, float]:
        """Compute the metrics from processed results.

        Args:
            results (list): The processed results of each batch.

        Returns:
            Dict[str, float]: The computed metrics. The keys are the names of
            the metrics, and the values are corresponding results.
        """
        logger: MMLogger = MMLogger.get_current_instance()

        clip_score_np = np.zeros(len(results))
        for i, result in enumerate(results):
            clip_score_np[i] = result['clip_sim_score']
        
        clip_sim_mean = np.mean(clip_score_np) 
        
        print("Test results: clip similarity score={:.4f}"
              .format(clip_sim_mean))

        return result



# Copyright (c) IFM Lab. All rights reserved.

import os
import cv2
import json
import torch
from torch.utils.data import Dataset
from torchvision import transforms
from PIL import Image
from core.registry import DATASETS
# import math

@DATASETS.register_module()
class SimpleVQADataset(Dataset):
    """
    Dataset for SimpleVQA.
    Each sample returns:
        - spatial_features (torch.Tensor): Extracted spatial frames.
        - motion_features (torch.Tensor): Extracted motion-based clips.
        - video_name (str): Video filename.
    """

    def __init__(self, video_dir, prompt_dir, min_video_seconds=8):
        super(SimpleVQADataset, self).__init__()
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        self.video_dir = video_dir
        self.prompt_dir = prompt_dir
        self.min_video_seconds = min_video_seconds

        self.prompts, self.video_names = self._read_prompt_videoname()
    
    def _read_prompt_videoname(self):
        with open(self.prompt_dir, 'r') as reader:
            read_data = json.load(reader)

        prompt_data_list, video_name_list = [], []
        for item in read_data["data_list"]:
            prompt = item['prompt_gt'].strip()
            video_name = item['video_path_pd'].strip()
            prompt_data_list.append(prompt)
            video_name_list.append(video_name)

        return prompt_data_list, video_name_list
    
    def __len__(self):
        return len(self.prompts)
    
    def video_processing_spatial(self, video_path):
        """
        Extracts spatial frames with proper resizing and normalization.
            - Key frame extraction: It selects 1 frame per second.
            - Standard input size: It resizes frames to 448 * 448 (after an initial resize to 520).
        Return:
            transformed_video (torch.Tensor): shape[video_length_read, 3, 448, 448]. 
                `video_length_read` is total seconds of the video (though 2 for toy dataset) with minium 8 (i.e. min_video_seconds).
            video_name (str)
        """
        video_capture = cv2.VideoCapture(video_path)
        video_name = os.path.basename(video_path)
        video_length = int(video_capture.get(cv2.CAP_PROP_FRAME_COUNT))
        video_frame_rate = int(round(video_capture.get(cv2.CAP_PROP_FPS)))

        # Compute the number of total seconds of the video
        video_length_read = int(video_length/video_frame_rate) # math.ceil()
        # print('video_length_read (s): ', video_length_read)
        transformations = transforms.Compose([
            transforms.Resize(520),
            transforms.CenterCrop(448),
            transforms.ToTensor(),
            transforms.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]) # Standard ImageNet normalization
        ])
        transformed_video = torch.zeros([max(video_length_read, self.min_video_seconds), 3, 448, 448])

        video_read_index = 0
        frame_idx = 0
        for i in range(video_length):
            has_frames, frame = video_capture.read()
            if has_frames:
                # Key frames extraction
                if (video_read_index < video_length_read) and (frame_idx % video_frame_rate == 0):
                    read_frame = Image.fromarray(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
                    read_frame = transformations(read_frame)
                    transformed_video[video_read_index] = read_frame
                    video_read_index += 1
                frame_idx += 1

        # Pads remaining frames by repeating the last available frame.
        if video_read_index < self.min_video_seconds:
            for i in range(video_read_index, self.min_video_seconds):
                transformed_video[i] = transformed_video[video_read_index - 1]

        video_capture.release()
        return transformed_video, video_name

    def video_processing_motion(self, video_path):
        """
        Extracts motion-based clips suitable for SlowFast.
            - Standard input size: It resizes frames to 224 * 224.
            - Motion-based clips: Processes at leaset 8-second clips, select 32 consecutive frames from each second.
        Return:
            transformed_video_all (List[torch.Tensor]): Tensor shape[video_length_clip(32), 3, 224, 224]. 
                Len(List) is total seconds of the video, with minium 8.
            video_name (str)
        """
        video_capture = cv2.VideoCapture(video_path)
        video_name = os.path.basename(video_path)
        video_length = int(video_capture.get(cv2.CAP_PROP_FRAME_COUNT))
        video_frame_rate = int(round(video_capture.get(cv2.CAP_PROP_FPS)))

        transform = transforms.Compose([
            transforms.Resize([224, 224]),
            transforms.ToTensor(),
            transforms.Normalize(mean=[0.45, 0.45, 0.45], std=[0.225, 0.225, 0.225]) # General purpose
        ])
        transformed_frame_all = torch.zeros([video_length, 3, 224, 224])
        video_read_index = 0
        for i in range(video_length): # All frames extraction
            has_frames, frame = video_capture.read()
            if has_frames:
                read_frame = Image.fromarray(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
                read_frame = transform(read_frame)
                transformed_frame_all[video_read_index] = read_frame
                video_read_index += 1

        # Pads remaining frames by repeating the last available frame.
        if video_read_index < video_length: 
            for i in range(video_read_index, video_length):
                transformed_frame_all[i] = transformed_frame_all[video_read_index - 1]
        
        video_capture.release()

        # Compute the number of total seconds of the video
        video_clip = int(video_length/video_frame_rate)
        # print('video_clip (s): ', video_clip)
        video_length_clip = 32
        transformed_video_all = []

        # Extract motion-based clips: select 32 consecutive frames from each second
        for i in range(video_clip):
            transformed_video = torch.zeros([video_length_clip, 3, 224, 224])
            if (i * video_frame_rate + video_length_clip) <= video_length: # if the clip can be fully extracted, select 32 consecutive frames starting at i*video_frame_rate
                transformed_video = transformed_frame_all[i * video_frame_rate:(i * video_frame_rate + video_length_clip)]
            else: # Copy all rest available frames. Pads remaining frames by repeating the last available frame.
                transformed_video[:(video_length - i * video_frame_rate)] = transformed_frame_all[i * video_frame_rate:]
                for j in range((video_length - i * video_frame_rate), video_length_clip):
                    transformed_video[j] = transformed_video[video_length - i * video_frame_rate - 1]
            transformed_video_all.append(transformed_video)

        if video_clip < self.min_video_seconds:
            for i in range(video_clip, self.min_video_seconds):
                transformed_video_all.append(transformed_video_all[video_clip - 1])
        
        return transformed_video_all, video_name

    def __getitem__(self, index):
        """
        Returns:
            spatial_features (torch.Tensor): Shape [v_len_second, 3, 448, 448]
                `v_len_second` is total seconds of the video (though 2 for toy dataset) with minium 8 (i.e. min_video_seconds).
            motion_features (List[torch.Tensor]): List of motion feature tensors.
                Each tensor has shape [32, 3, 224, 224].
                Len(List) is total seconds of the video (i.e. v_len_second), with minium 8 (i.e. min_video_seconds).
            video_name (str): Video filename
        """
        video_name = self.video_names[index]
        video_path = os.path.join(self.video_dir, video_name)

        spatial_features, video_name = self.video_processing_spatial(video_path)
        motion_features, video_name = self.video_processing_motion(video_path)
        # print('spatial_features: ', spatial_features.shape) # torch.Size([8, 3, 448, 448]) for toy dataset
        # print('motion_features len: ', len(motion_features)) # 8
        # print('motion_features[0]: ', motion_features[0].shape) # torch.Size([32, 3, 224, 224])

        return spatial_features, motion_features, video_name



# Copyright 2020 The HuggingFace Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import TYPE_CHECKING

from transformers.utils import OptionalDependencyNotAvailable, _LazyModule, is_tokenizers_available, is_torch_available


_import_structure = {
    "configuration_mplug_owl": ["MPLUG_OWL_PRETRAINED_CONFIG_ARCHIVE_MAP", "MplugOwlConfig"],
    "processing_mplug_owl": ["MplugOwlImageProcessor", "MplugOwlProcessor"],
    "tokenization_mplug_owl": ["MplugOwlTokenizer"],
}

try:
    if not is_tokenizers_available():
        raise OptionalDependencyNotAvailable()
except OptionalDependencyNotAvailable:
    pass


try:
    if not is_torch_available():
        raise OptionalDependencyNotAvailable()
except OptionalDependencyNotAvailable:
    pass
else:
    _import_structure["modeling_mplug_owl"] = [
        "MPLUG_OWL_PRETRAINED_MODEL_ARCHIVE_LIST",
        "MplugOwlForConditionalGeneration",
        "MplugOwlModel",
    ]


if TYPE_CHECKING:
    from .configuration_mplug_owl import MPLUG_OWL_PRETRAINED_CONFIG_ARCHIVE_MAP, MplugOwlConfig
    from .tokenization_mplug_owl import MplugOwlTokenizer

    try:
        if not is_tokenizers_available():
            raise OptionalDependencyNotAvailable()
    except OptionalDependencyNotAvailable:
        pass

    try:
        if not is_torch_available():
            raise OptionalDependencyNotAvailable()
    except OptionalDependencyNotAvailable:
        pass
    else:
        from .modeling_mplug_owl import (
            MPLUG_OWL_PRETRAINED_MODEL_ARCHIVE_LIST,
            MplugOwlForConditionalGeneration,
            MplugOwlModel,
            MplugOwlPreTrainedModel,
        )


else:
    import sys

    sys.modules[__name__] = _LazyModule(__name__, globals()["__file__"], _import_structure, module_spec=__spec__)

from .configuration_mplug_owl import *
from .modeling_mplug_owl import *
from .processing_mplug_owl import *
from .tokenization_mplug_owl import *



# Copyright (c) IFM Lab. All rights reserved.

# from .modular_bvqa_metric import ModularBVQA
# rom .GSTVQA.TCSVT_Release.GVQA_Release.GVQA_Cross.cross_test import Test_VQADataset

# __all__ = ['ModularBVQA']


# Copyright (c) IFM Lab. All rights reserved.

from mmengine.registry import Registry
from mmengine.registry import EVALUATOR as MMENGINE_EVALUATORS
from mmengine.registry import RUNNERS as MMENGINE_RUNNERS
from mmengine.registry import LOOPS as MMENGINE_LOOPS
from mmengine.registry import METRICS as MMENGINE_METRICS
from mmengine.registry import TRANSFORMS as MMENGINE_TRANSFORMS
from mmengine.registry import DATASETS as MMENGINE_DATASETS
from mmengine.registry import MODELS as MMENGINE_MODELS

LOOPS = Registry('loop', parent=MMENGINE_LOOPS, locations=['core.loops'])
DATASETS = Registry('dataset', parent=MMENGINE_DATASETS, locations=['datasets'])
TRANSFORMS = Registry('transform', parent=MMENGINE_TRANSFORMS, locations=['transforms'])
METRICS = Registry('metric', parent=MMENGINE_METRICS, locations=['metrics'])
MODELS = Registry('model', parent=MMENGINE_MODELS, locations=['core.models'])




# Copyright (c) IFM Lab. All rights reserved.

import subprocess, os

def add_git_submodule(repo_url, submodule_path):
    try:
        subprocess.run(['git', 'submodule', 'add', repo_url, submodule_path], check=True)
        subprocess.run(['git', 'submodule', 'update', '--init', '--recursive'], check=True)
        print(f'Successfully added submodule {repo_url} at {submodule_path}')
    except subprocess.CalledProcessError as e:
        print(f'An error occured while adding the submodule {repo_url} at {submodule_path}: {e}')

def submodule_exists(submodule_path):
    if os.path.isfile('.gitmodules'):
        with open('.gitmodules', 'r') as file:
            if submodule_path in file.read():
                return True
    return False